@@ -37,6 +37,7 @@ from multiprocessing.pool import ThreadPool
 from robot.run import USAGE
 from robot.utils import ArgumentParser
 import signal
+import PabotLib
 from result_merger import merge
 import Queue
 
@@ -59,7 +60,7 @@ def execute_and_wait_with(args):
         open(os.path.join(outs_dir, 'stderr.txt'), 'w') as stderr:
             process, rc = _run(cmd, stderr, stdout, suite_name, verbose)
     if rc != 0:
-        _write(_execution_failed_message(suite_name, process, rc, verbose))
+        _write(_execution_failed_message(suite_name, rc, verbose))
     else:
         _write('PASSED %s' % suite_name)
 
@@ -82,7 +83,7 @@ def _run(cmd, stderr, stdout, suite_name, verbose):
             _write('[PID:%s] still running %s after %s seconds' % (process.pid, suite_name, elapsed / 10.0))
     return process, rc
 
-def _execution_failed_message(suite_name, process, rc, verbose):
+def _execution_failed_message(suite_name, rc, verbose):
     if not verbose:
         return 'FAILED %s' % suite_name
     return 'Execution failed in %s with %d failing test(s)' % (suite_name, rc)
@@ -142,8 +143,10 @@ def get_suite_names(output_file):
 def _parse_args(args):
     pabot_args = {'command':['pybot'],
                   'verbose':False,
+                  'pabotlib':False,
                   'processes':max(multiprocessing.cpu_count(), 2)}
-    while args and args[0] in ['--command', '--processes', '--verbose']:
+    while args and args[0] in ['--'+param for param in ['command', 'processes', 'verbose', 'resourcefile',
+                                                        'pabotlib']]:
         if args[0] == '--command':
             end_index = args.index('--end-command')
             pabot_args['command'] = args[1:end_index]
@@ -154,6 +157,12 @@ def _parse_args(args):
         if args[0] == '--verbose':
             pabot_args['verbose'] = True
             args = args[1:]
+        if args[0] == '--resourcefile':
+            pabot_args['resourcefile'] = args[1]
+            args = args[2:]
+        if args[0] == '--pabotlib':
+            pabot_args['pabotlib'] = True
+            args = args[1:]
     options, datasources = ArgumentParser(USAGE, auto_pythonpath=False, auto_argumentfile=False).parse_args(args)
     keys = set()
     for k in options:
@@ -220,25 +229,24 @@ def keyboard_interrupt(*args):
     CTRL_C_PRESSED = True
 
 def _parallel_execute(datasources, options, outs_dir, pabot_args, suite_names):
-    if suite_names:
-        original_signal_handler = signal.signal(signal.SIGINT, keyboard_interrupt)
-        pool = ThreadPool(pabot_args['processes'])
-        result = pool.map_async(execute_and_wait_with,
-                   [(datasources,
-                     outs_dir,
-                     options,
-                     suite,
-                     pabot_args['command'],
-                     pabot_args['verbose'])
-                    for suite in suite_names])
-        pool.close()
-        while not result.ready():
-            # keyboard interrupt is executed in main thread and needs this loop to get time to get executed
-            try:
-                time.sleep(0.1)
-            except IOError:
-                keyboard_interrupt()
-        signal.signal(signal.SIGINT, original_signal_handler)
+    original_signal_handler = signal.signal(signal.SIGINT, keyboard_interrupt)
+    pool = ThreadPool(pabot_args['processes'])
+    result = pool.map_async(execute_and_wait_with,
+               [(datasources,
+                 outs_dir,
+                 options,
+                 suite,
+                 pabot_args['command'],
+                 pabot_args['verbose'])
+                for suite in suite_names])
+    pool.close()
+    while not result.ready():
+        # keyboard interrupt is executed in main thread and needs this loop to get time to get executed
+        try:
+            time.sleep(0.1)
+        except IOError:
+            keyboard_interrupt()
+    signal.signal(signal.SIGINT, original_signal_handler)
 
 def _output_dir(options):
     outputdir = '.'
@@ -269,17 +277,32 @@ def _start_message_writer():
     t.setDaemon(True)
     t.start()
 
+def _start_remote_library(pabot_args):
+    if not pabot_args['pabotlib']:
+        return None
+    return subprocess.Popen('python %s %s' % (os.path.abspath(PabotLib.__file__),
+                                              pabot_args.get('resourcefile', 'N/A')),
+                            shell=True)
+
+def _stop_remote_library(process):
+    process.terminate()
+
 def main(args):
     start_time = time.time()
     start_time_string = _now()
+    lib_process = None
     #NOTE: timeout option
     try:
         _start_message_writer()
         options, datasources, pabot_args = _parse_args(args)
+        lib_process = _start_remote_library(pabot_args)
         outs_dir = _output_dir(options)
         suite_names = solve_suite_names(outs_dir, datasources, options)
-        _parallel_execute(datasources, options, outs_dir, pabot_args, suite_names)
-        sys.exit(_report_results(outs_dir, options, start_time_string))
+        if suite_names:
+            _parallel_execute(datasources, options, outs_dir, pabot_args, suite_names)
+            sys.exit(_report_results(outs_dir, options, start_time_string))
+        else:
+            print 'No tests to execute'
     except Information, i:
         print """A parallel executor for Robot Framework test cases.
 
@@ -294,10 +317,18 @@ RF script for situations where pybot is not used directly
 --processes [NUMBER OF PROCESSES]
 How many parallel executors to use (default max of 2 and cpu count)
 
+--resourcefile [FILEPATH]
+Indicator for a file that can contain shared variables for distributing resources.
+
+--pabotlib
+Start PabotLib remote server. This enables locking and resource distribution between parallel test executions.
+
 Copyright 2014 Mikko Korpela - GPLv3
 """
         print i.message
     finally:
+        if lib_process:
+            _stop_remote_library(lib_process)
         _print_elapsed(start_time, time.time())
 
 
